<?php

namespace Nwb\UrlToPdf;

class UrlToPdf
{
    /**
     * @var string
     */
    private string $apiUrl = 'https://url-to-pdf.kub.nwb.fr';

    /**
     * @var string|null
     */
    private ?string $accessKey;

    /**
     * UrlToPdf constructor.
     * @param array $options Liste des options de configuration
     * @throws Exception
     */
    public function __construct(array $options = [])
    {
        $key = $options['accessKey'] ?? $_ENV['URL_TO_PDF_ACCESS_KEY'] ??  null;

        if (!$key) {
            throw new \Exception('Missing access key');
        }

        $this->accessKey = $key;

        $this->apiUrl = $options['apiUrl'] ?? $_ENV['URL_TO_PDF_APIURL'] ?? $this->apiUrl;
    }

    /**
     * Requête curl vers l'API
     * 
     * @param string $url contient la fin de l'url pour appeler la bonne méthode de l'API (ex : /get-pdf)
     * @param array $options Liste des options de configuration, notamment $options['url'] l'url vers la page html à convertir en PDF
     * @return false|resource contenu du PDF
     * @throws Exception si la page est vide
     * @throws Exception si le code HTTP n'est pas 200
     * @throws Exception si le contenu de la réponse n'est pas un PDF
     * @throws Exception si le contenu de la réponse est vide
     */
    private function request(string $url, array $options)
    {
        // Check options
        if (empty($options['url'])) {
            throw new \Exception('Missing url');
        }

        // On construit l'url de l'HTML à transmettre à l'API si authentification basique au server (ex: https://username:password@url)
        if (!empty($options['auth']) && !empty($options['auth']['username']) && !empty($options['auth']['password'])) {
            $username = $options['auth']['username'];
            $password = $options['auth']['password'];
            $options['url'] = str_replace('://', '://' . $username . ':' . $password . '@', $options['url']);
        }

        // Init 1er curl pour récupérer le contenu de la page html, permet de tester si l'url est valide et si le contenu est bien du html
        $ch1 = curl_init();
        curl_setopt($ch1, CURLOPT_URL, $options['url']);
        curl_setopt($ch1, CURLOPT_RETURNTRANSFER, true);

        $pageContent = curl_exec($ch1);

        // Erreur cURL
        if ($pageContent === false) {
            throw new \Exception("Erreur cURL lors de la récupération du fichier html : " . curl_error($ch1));
        }

        // Vérifier si la page est vide
        if (empty($pageContent)) {
            throw new \Exception("La page html récupérée n'a pas de contenu.");
        }

        // Vérifier si le code HTTP est 200
        $httpCode = curl_getinfo($ch1, CURLINFO_HTTP_CODE);
        $contentType = curl_getinfo($ch1, CURLINFO_CONTENT_TYPE);
        if ($httpCode !== 200) {
            if ($pageContent) {
                throw new \Exception("Une erreur s'est produite lors de la vérification du contenu du fichier html : " . $pageContent);
            }

            throw new \Exception('Error: ' . $httpCode);
        }

        curl_close($ch1);

        // Init 2ème curl pour appeler l'API
        $ch = curl_init();

        // Définition des headers
        $headers = [
            'Content-Type: application/json',
            'ACCESS-KEY: ' . $this->accessKey,
        ];

        curl_setopt($ch, CURLOPT_URL, $this->apiUrl . $url);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($options));
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $response = curl_exec($ch);

        // Erreur cURL
        if ($response === false) {
            throw new \Exception("Erreur cURL : " . curl_error($ch));
        }

        // Vérifier si la réponse est vide
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        $contentType = curl_getinfo($ch, CURLINFO_CONTENT_TYPE);
        if ($httpCode !== 200) {
            if (strpos($contentType, 'application/json') !== false) {
                $errorText = json_decode($response, true);

                if (count($errorText)) {
                    throw new \Exception('Erreur de l\'API : ' . ($errorText->message ?? $errorText['message'] ?? print_r($errorText, true)));
                }
            } else {
                if ($response) {
                    throw new \Exception('Erreur cURL : ' . $response);
                }
            }

            throw new \Exception('Code d\'erreur http : ' . $httpCode);
        }

        if (empty($response) || substr($response, 0, 4) !== "%PDF") {
            throw new \Exception('La réponse retournée est vide ou n\'est pas un PDF.');
        }

        return $response;
    }

    /**
     * Enregistre le contenu dans un fichier sur le disque.
     *
     * @param string $content Contenu à enregistrer.
     * @param string $destination Chemin de destination du fichier.
     * @return bool
     */
    public function saveTo(string $content, string $destination): bool
    {
        // Écriture du contenu dans le fichier
        return file_put_contents($destination, $content) !== false;
    }

    /**
     * Retourne un PDF à partir d'une URL et des options
     * 
     * @param array $options Liste des options de configuration
     * @return false|resource contenu du PDF
     */
    public function getPdf(?array $options)
    {
        $options = $options ?? [
            'url' => 'https://normandiewebschool.fr/cursus/bachelor-developpement-web/',
            'urlMustHave200' => false,
            'loadingOptions' => ['waitUntil' => 'networkidle2'],
            'pdfOptions' => ['format' => 'a4']
        ];

        return $this->request('/get-pdf', $options);
    }

    
    private function extractAndCheckBackgroundUrls(string $html):array
    {
        // Regex pour trouver les URL dans les propriétés CSS background et background-image
        $regex = '/background(?:-image)?:\s*url\((["\']?)(.*?)\1\)/i';

        // Tableau pour stocker les URL extraites
        $uniqueUrls = [];

        // Exécuter la regex
        if (preg_match_all($regex, $html, $matches)) {
            foreach ($matches[2] as $url) {
                $url = trim($url);  // Nettoyer les espaces avant et après l'URL
                if (!in_array($url, $uniqueUrls)) {  // Vérifier si l'URL n'est pas déjà ajoutée
                    $uniqueUrls[] = $url;
                }
            }
        }

        return $uniqueUrls;
    }

    /**
     * Retourne un PDF à partir d'un contenu HTML
     * 
     * @param string $htmlContent type TCPDF qui peut contenir des liens vers des images/fichiers sur le disque (ex: src:"/var/www/html/public-data/tmp/url_to_pdf/mon_image.jpg")
     * @param string $projectRootPath Chemin racine du projet (ex: /var/www/html/)
     * @param string $webPath Chemin vers le répertoire web (ex: /var/www/html/public-data/tmp/url_to_pdf/)
     * @param string $webPathUrl Chemin vers le répertoire web accessible depuis l'API (ex: https://www.nwb.fr/public-data/tmp/url_to_pdf/)
     * @param array $loadingOptions Liste des options de configuration (waitUntil, timeout, etc.). Par défaut : ['waitUntil' => 'networkidle2', 'timeout' => 30000]
     * @param array $pdfOptions Liste des options de configuration (marges, format, etc.). Par défaut : ['format' => 'a4', 'printBackground' => true, 'margin' => ['top' => '0.5cm', 'right' => '0.5cm', 'bottom' => '0.5cm', 'left' => '0.5cm']]
     * @param array $authOptions Liste des options d'authentification (username, password). Utile pour une preprod par exemple.
     * @param bool $isTestMode Indique si on est en mode test, dans ce cas on utilise l'API pour envoyer le fichier dans l'url en Base64
     * @return false|string chemin du PDF
     * @throws Exception
     * 
     */
    public function getPdfFromHtml(string $htmlContent, string $projectRootPath, string $webPath, string $webPathUrl, array $loadingOptions = [], array $pdfOptions = [], array $authOptions = [], bool $isTestMode = false): string
    {
        // Test si $htmlContent est vide, si oui on throw une exception
        if (!$htmlContent) {
            throw new \Exception("Le contenu HTML est vide.");
        }

        // Si répertoire web n'existe pas on trhow une exception
        if (!is_dir($webPath)) {
            throw new \Exception("Le répertoire web n'existe pas : " . $webPath);
        }

        // Test si $webPathUrl existe et fait plus de 8 caractères, si non on throw une exception
        if (!$webPathUrl || strlen($webPathUrl) < 8) {
            throw new \Exception("Le chemin web n'est pas valide : " . $webPathUrl);
        }

        // Test si $webPath se termine par un slash, si non on le rajoute
        if (substr($webPath, -1) !== '/') {
            $webPath .= '/';
        }

        // Test si $webPathUrl se termine par un slash, si non on le rajoute
        if (substr($webPathUrl, -1) !== '/') {
            $webPathUrl .= '/';
        }

        // Test si $webPathUrl ne commence pas par https:// ou http:// , si non on le rajoute
        $webPathUrl = trim($webPathUrl);
        if (strpos($webPathUrl, 'https://', 0) !== 0 && strpos($webPathUrl, 'http://', 0) !== 0) {
            $webPathUrl = ($isTestMode) ? 'http://' . $webPathUrl : 'https://' . $webPathUrl;
        }

        // Garde le webpath en mémoire pour le retour du pdf final
        $webPathFinal = $webPath;

        // On hash le contenu html pour créer un nom de dossier temporaire unique
        $name_dir = md5($htmlContent);
        $tempWebPath = $webPath . $name_dir . '/';
        $tempWebPathUrl = $webPathUrl . $name_dir . '/';

        // Check options PDF et définition des options de base Puppeteer par défaut
        $pdfOptions = $pdfOptions ?? [
            'format' => 'a4',
        ];

        // On check les options dédiées au module
        $pdfOptions['noBreakTableRows'] = $pdfOptions['noBreakTableRows'] ?? true;

        // Check option urlMustHave200 permettant de retourner une erreur si l'URL de destination a un status code différent de 200
        $pdfOptions['urlMustHave200'] = $pdfOptions['urlMustHave200'] ?? false;

        // Adapte le html pour accessibilité web 
        $htmlContent = $this->getCleanHtml($htmlContent, $projectRootPath, $tempWebPath, $tempWebPathUrl, $pdfOptions, $isTestMode);

        // On hash le contenu html pour créer un nom de fichier unique
        $fileName = 'pdf.html';

        // On créer le fichier
        $filePath = $tempWebPath . $fileName;
        $file = fopen($filePath, 'w');

        // Enregistre le html dans un fichier temporaire
        fwrite($file, $htmlContent);
        fclose($file);

        // On met les droits en 777
        chmod($filePath, 0777);

        // Ajout des options de chargement de la page
        $loadingOptions = $loadingOptions ?? [
            'waitUntil' => 'networkidle2',
            'timeout' => 30000
        ];

        // Appel de l'API getPdf
        if (!$isTestMode) {
            $pdf = $this->getPdf([
                'url' => $tempWebPathUrl . $fileName,
                'urlMustHave200' => $pdfOptions['urlMustHave200'],
                'loadingOptions' => $loadingOptions,
                'pdfOptions' => $pdfOptions,
                'auth' => $authOptions
            ]);
        }

        // Si on est en mode test on passe le contenu du fichier en base64 dans l'url (valable uniquement pour les petits fichiers)
        if ($isTestMode) {
            // On enregistre le html dans un fichier temporaire
            fopen($webPath . 'TEST_' . $fileName, 'w');
            file_put_contents($webPath . 'TEST_' . $fileName, $htmlContent);

            $htmlContent = base64_encode($htmlContent);

            // Troncature pour que le html content fasse moins de 3000 caractères (limite du GET)
            $htmlContent = substr($htmlContent, 0, 3000);

            // Appel de getPdf avec une url générée par l'API BASE64ToHtml
            $pdf = $this->getPdf([
                'url' => 'https://n8n.kub.nwb.fr/webhook/Base64ToHtml?htmlB64=' . $htmlContent,
                'pdfOptions' => $pdfOptions
            ]);
        }

        // Remplacement du contenu du fichier temporaire par le contenu du PDF et on renomme le fichier en .pdf
        $file = fopen($filePath, 'w');
        fwrite($file, $pdf);
        fclose($file);
        rename($filePath, $tempWebPath . pathinfo($filePath, PATHINFO_FILENAME) . '.pdf');
        $filePath = $tempWebPath . pathinfo($filePath, PATHINFO_FILENAME) . '.pdf';

        // copie le fichier dans le répertoire final
        $hashFileName = md5_file($filePath);
        $filePathReturn = $webPathFinal . $hashFileName . '.pdf';
        copy($filePath, $filePathReturn);

        // On supprime les imafges temporaires sauf si on est en mode test
        if (!$isTestMode) {
            $this->deleteTempImages($tempWebPath);
        }

        // Retourne le path temporaire du PDF
        return $filePathReturn;
    }

    private function getImageInfo($url)
    {
        // Initialise cURL
        $ch = curl_init();

        // Set the URL and other options
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_NOBODY, true); // We don't need the body, only the headers
        curl_setopt($ch, CURLOPT_HEADER, true); // Include the headers in the output
        curl_setopt($ch, CURLOPT_TIMEOUT, 10); // Timeout after 10 seconds

        // Execute the request
        $response = curl_exec($ch);

        // Get the HTTP response code
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        $contentType = null;

        // Get the content type from the headers if the response is successful
        if ($httpCode == 200) {
            if (preg_match('/Content-Type:\s([a-zA-Z0-9\/\-\.]+)/i', $response, $matches)) {
                $contentType = trim($matches[1]);
            }
        }

        // Close cURL
        curl_close($ch);

        if ($httpCode != 200 || !$contentType) {
            throw new \Exception("Impossible d'accéder à cette image : " . $url);
        }

        return [
            'http_code' => $httpCode,
            'content_type' => $contentType
        ];
    }

    private function getExtensionFromContentType($contentType)
    {
        $mimeTypes = [
            'image/jpeg' => 'jpg',
            'image/png' => 'png',
            'image/gif' => 'gif',
            'image/bmp' => 'bmp',
            'image/webp' => 'webp'
            // Add other MIME types and their extensions as needed
        ];

        return isset($mimeTypes[$contentType]) ? $mimeTypes[$contentType] : null;
    }


    /**
     * Retourne le html passé en paramètre avec les path des fichiers accessibles pour l'API Url to PDF
     * 
     * @param string $htmlIn
     * @param string $projectRootPath (ex: /var/www/html/)
     * @param string $webPath (ex: /var/www/html/public-data/tmp/url_to_pdf/)
     * @param string $webPathUrl (ex: https://www.nwb.fr/public-data/tmp/url_to_pdf/)
     * @param array $pdfOptions Liste des options de configuration dédiées au module et à la conversion html
     * @param bool $isTestMode Indique si on est en mode test, dans ce cas on ne n'insère pas les path sous forme d'url
     * @return string $htmlContent
     * @throws Exception si l'image n'existe pas
     */
    public function getCleanHtml(string $htmlIn, string $projectRootPath, string $webPath, string $webPathUrl, array $pdfOptions = [], bool $isTestMode = false): string
    {
        if (!$htmlIn) {
            throw new \Exception("Le contenu HTML est vide.");
        }
        if (!$projectRootPath) {
            throw new \Exception("Le chemin racine du projet n'est pas renseigné.");
        }
        if (!$webPath) {
            throw new \Exception("Le chemin vers le répertoire web n'est pas renseigné.");
        }
        if (!$webPathUrl) {
            throw new \Exception("L'url vers le répertoire web n'est pas renseigné.");
        }

        // Test si $webPath se termine par un slash, si non on le rajoute
        if (substr($webPath, -1) !== '/') {
            $webPath .= '/';
        }

        // Test si $webPathUrl se termine par un slash, si non on le rajoute
        if (substr($webPathUrl, -1) !== '/') {
            $webPathUrl .= '/';
        }

        // On créer le répertoire s'il n'existe pas
        if (is_dir($webPath)) {
            $this->deleteDir($webPath);
        }
        if (!mkdir($webPath, 0777, true)) {
            throw new \Exception("Le répertoire web n'a pas pu être créé.");
        }

        // On récupère le contenu HTML
        $htmlContent = $htmlIn;

        // Intit le dom HTML
        $dom = new \DOMDocument();

        // On désactive les erreurs libxml (utilisé par domdocument)
        libxml_use_internal_errors(true);

        // On charge le contenu HTML
        $dom->loadHTML($htmlContent);

        // On définit les tags et attributs à traiter
        $tagAttributes = [
            'img' => 'src',
            'script' => 'src',
            'link' => 'href',
            'page_header' => '',
            'page_footer' => '',
            'page' => '',
        ];

        $compteurNomImage = 0;

        // On parcourt tous les éléments à traiter
        foreach ($tagAttributes as $tag => $attribute) {
            $elements = $dom->getElementsByTagName($tag);
            if ($elements->length === 0) {
                continue;
            }

            foreach ($elements as $element) {
                // On traite d'abord les balises <page> <page_header> et <page_footer> provenant de la librairie TCPDF
                if ($tag === 'page' || $tag === 'page_header' || $tag === 'page_footer') {
                    // Conversion des balises en div 
                    $newElement = $dom->createElement('div');
                    $newElement->setAttribute('id', $tag);
                    $newElement->setAttribute('class', $tag);
                    if (!empty($element->getAttribute('style'))) {
                        $newElement->setAttribute('style', $element->getAttribute('style'));
                    }
                    // Si la balise à des attributs connus comme la gestion des marges on les convertit en style
                    $attributes = $element->attributes;
                    foreach ($attributes as $attrName => $attrNode) {
                        if ($attrName === 'backtop' || $attrName === 'backbottom' || $attrName === 'backleft' || $attrName === 'backright') {
                            switch ($attrName) {
                                case 'backtop':
                                    $newElement->setAttribute('style', $newElement->getAttribute('style') . 'margin-top:' . $attrNode->nodeValue . ';');
                                    break;
                                case 'backbottom':
                                    $newElement->setAttribute('style', $newElement->getAttribute('style') . 'margin-bottom:' . $attrNode->nodeValue . ';');
                                    break;
                                case 'backleft':
                                    $newElement->setAttribute('style', $newElement->getAttribute('style') . 'margin-left:' . $attrNode->nodeValue . ';');
                                    break;
                                case 'backright':
                                    $newElement->setAttribute('style', $newElement->getAttribute('style') . 'margin-right:' . $attrNode->nodeValue . ';');
                                    break;
                            }
                        }
                    }
                    // On déplace les enfants de la balise dans la nouvelle balise
                    while ($element->firstChild) {
                        $newElement->appendChild($element->firstChild);
                    }
                    $element->parentNode->replaceChild($newElement, $element);
                }

                $filePath = $element->getAttribute($attribute);

                // Test si le chemin est vide
                if (!$filePath) {
                    continue;
                }

                $formatPath = "/^(https?):\/\/(?![^\/]*\.s3)([\w\-\.]+)(\.[a-z\.]{2,6})(:[0-9]+)?([\/\w \.-]*)*(\?[\/\w \.-=&%-]*)?\/?$/";
                if (preg_match($formatPath, $filePath)) {

                    $fileExist = $this->getImageInfo($filePath);
                    $extension = $this->getExtensionFromContentType($fileExist['content_type']);
                    if (!$extension) {
                        throw new \Exception("Ce type de fichier n'est pas supporté.");
                    }

                    $imageData = @file_get_contents($filePath);

                    if ($imageData === false) {
                        throw new \Exception("Le fichier n'a pas pu être récupérer.");
                    }

                    $newBasename = 'url_to_pdf_image_' . $compteurNomImage . '.' . $extension;
                    $compteurNomImage++;

                    // Save the image data to the destination path
                    if (@file_put_contents($webPath . $newBasename, $imageData) === false) {
                        throw new \Exception("Une erreur s'est produite lors de la copie de l'image");
                    }

                    $element->setAttribute($attribute, $newBasename);
                    continue;
                }

                $formatPath = "/^(https?):\/\/([\w\-\.]+\.s3)(\.[a-z\.]{2,6})(:[0-9]+)?([\/\w \.-]*)*(\?[\/\w \.-=&%-]*)?\/?$/";
                if (preg_match($formatPath, $filePath)) {
                    continue;
                }

                // Si le chemin contient ./ ou ../ ou // on les supprime
                $filePath = str_replace('../', '', $filePath);
                $filePath = str_replace('./', '', $filePath);


                // Si le chemin est une url ou un protocole de fichier ftp ou s3 en localhost on supprime la partie protocole et domaine
                $formatPath = "/^(https?|ftp|s3):\/\/(localhost|[\da-z\.-]+)(:[0-9]+)?(\.[a-z\.]{2,6})?([\/\w \.-]*)*\/?$/";
                if (preg_match($formatPath, $filePath)) {
                    $filePath = parse_url($filePath, PHP_URL_PATH);
                }

                // Si un '//' est présent on le supprime
                $filePath = str_replace('//', '/', $filePath);

                // Si le chemin commence par un slash et contient le pathRoot on le supprime
                if (strpos($filePath, '/', 0) === 0 && strpos($filePath, $projectRootPath, 0) !== 0) {
                    $filePath = substr($filePath, 1);
                }

                // Test du chemin réel
                if (realpath($filePath)) {
                    $filePath = realpath($filePath);
                }

                // On ajoute le chemin racine du projet s'il n'est pas déjà présent
                if (strpos($filePath, $projectRootPath, 0) !== 0) {
                    $filePath = $projectRootPath . '/' . $filePath;
                }

                // On test si le fichier existe
                if (!file_exists($filePath)) {
                    throw new \Exception("Le fichier n'existe pas : " . $filePath);
                }

                // On créer un nouveau basename pour éviter les doublons (uniquement si on est pas en mode test)
                $newBasename = 'url_to_pdf_image_' . $compteurNomImage . '.' . pathinfo($filePath, PATHINFO_EXTENSION);
                $compteurNomImage++;

                if ($isTestMode) {
                    $newBasename = pathinfo($filePath, PATHINFO_BASENAME);
                }

                // Nouveau chemin absolu
                $newFilePath = $webPath . $newBasename;

                // On copie le fichier dans le nouveau répertoire
                if (!copy($filePath, $newFilePath)) {
                    throw new \Exception("Le fichier n'a pas pu être copié.");
                }
                // On met les droits en 777
                chmod($newFilePath, 0777);

                // On remplace le chemin dans l'attribut de l'élément par le chemin accessible depuis l'API
                $formatPath = "/^(https?|ftp|s3):\/\/(localhost|[\da-z\.-]+)(\.[a-z\.]{2,6})?(:\d{2,5})?([\/\w \.-]*)*\/?$/";
                $newRelativePath = $webPathUrl . $newBasename;
                if (!preg_match($formatPath, $newRelativePath)) {
                    throw new \Exception("L'url n'est pas valide pour : " . $newRelativePath);
                }
                $element->setAttribute($attribute, $newBasename);
            }
        }

        // On sauvegarde le nouveau contenu HTML 
        $htmlContent = $dom->saveHTML();

        $backgroundUrls = $this->extractAndCheckBackgroundUrls($htmlContent);
        if (substr($projectRootPath, -1) !== '/') {
            $projectRootPath .= '/';
        }
        foreach ($backgroundUrls as $backgroundImage) {

            $formatPath = "/^(https?|ftp|s3|file|gopher|ws|wss):\/\/([\w\-\.]+)(\.[a-z\.]{2,6})(:[0-9]+)?([\/\w \.-]*)*(\?[\/\w \.-=&%-]*)?\/?$/";
            if (preg_match($formatPath, $backgroundImage)) {

                // On récupère les informations de l'image / Si impossible la fonction renvoie une exception
                $fileExist = $this->getImageInfo($backgroundImage);

                // On récupère l'extension de l'image
                $extension = $this->getExtensionFromContentType($fileExist['content_type']);
                // Si l'extension n'est pas supportée on renvoie une exception
                if (!$extension) {
                    throw new \Exception("Ce type de fichier n'est pas supporté.");
                }

                // On récupère le contenu de l'image
                $imageData = @file_get_contents($backgroundImage);

                // Si impossible de récupérer l'image on renvoie une exception
                if ($imageData === false) {
                    throw new \Exception("Le fichier n'a pas pu être récupérer.");
                }

                $newBasename = 'url_to_pdf_image_' . $compteurNomImage . '.' . $extension;
                $compteurNomImage++;

                // On sauvegarde le contenu de l'image dans le répertoire web
                if (@file_put_contents($webPath . $newBasename, $imageData) === false) {
                    throw new \Exception("Une erreur s'est produite lors de la copie de l'image");
                }

                chmod($webPath . $newBasename, 0777);
                $htmlContent = str_replace($backgroundImage, $newBasename, $htmlContent);
                continue;
            }

            $filePath = str_replace('//', '/', $backgroundImage);

            // Si le chemin commence par un slash et contient le pathRoot on le supprime
            if (strpos($filePath, '/', 0) === 0 && strpos($filePath, $projectRootPath, 0) !== 0) {
                $filePath = substr($filePath, 1);
            }

            // Test du chemin réel
            if (realpath($filePath)) {
                $filePath = realpath($filePath);
            }

            // On ajoute le chemin racine du projet s'il n'est pas déjà présent
            if (strpos($filePath, $projectRootPath, 0) !== 0) {
                $filePath = $projectRootPath . '/' . $filePath;
            }

            // On test si le fichier existe
            if (!file_exists($filePath)) {
                throw new \Exception("Le fichier n'existe pas : " . $filePath);
            }

            $newBasename = 'url_to_pdf_image_' . $compteurNomImage . '.' . pathinfo($filePath, PATHINFO_EXTENSION);
            $compteurNomImage++;

            // On copie le fichier dans le nouveau répertoire
            if (!copy($filePath, $webPath . $newBasename)) {
                throw new \Exception("Le fichier n'a pas pu être copié.");
            }
            // On met les droits en 777
            chmod($webPath . $newBasename, 0777);
            $htmlContent = str_replace($backgroundImage, $newBasename, $htmlContent);
        }


        // Charge les options du module
        $styleAttributes = '';
        if (isset($pdfOptions['noBreakTableRows']) && $pdfOptions['noBreakTableRows']) {
            $styleAttributes .= 'tr, td, th { page-break-inside: avoid; }';
        }
        if (isset($pdfOptions['font'])) {
            $styleAttributes .= 'html { ';
            foreach ($pdfOptions['font'] as $font => $att) {
                $styleAttributes .= 'font-' . $font . ': ' . $att . '; ';
            }
            $styleAttributes .= '}';
        }

        // Si pas d'options de module on retourne le contenu HTML
        if (empty($styleAttributes)) {
            return $htmlContent;
        }

        // Recharge le contenu HTML
        $dom->loadHTML($htmlContent);

        // Si options module on ajoute le style dans le head
        if (!empty($styleAttributes) && $dom->getElementsByTagName('style')->length === 0) { // Si pas de balise style on la créer
            $style = $dom->createElement('style', $styleAttributes);
        }
        if (!empty($styleAttributes) && $dom->getElementsByTagName('style')->length > 0) { // Si balise style on ajoute les attributs
            $style = $dom->getElementsByTagName('style')->item(0);
            $style->nodeValue .= $styleAttributes;
        }
        if (!empty($styleAttributes) && $dom->getElementsByTagName('head')->length > 0) { // Si balise head on ajoute la balise style
            $dom->getElementsByTagName('head')->item(0)->appendChild($style);
        }
        if (!empty($styleAttributes) && $dom->getElementsByTagName('head')->length === 0) { // Si pas de balise head on la créer
            $head = $dom->createElement('head');
            if ($dom->getElementsByTagName('html')->item(0)->hasChildNodes()) {
                $firstChild = $dom->getElementsByTagName('html')->item(0)->firstChild;
                $dom->getElementsByTagName('html')->item(0)->insertBefore($head, $firstChild);
            } else {
                $dom->getElementsByTagName('html')->item(0)->appendChild($head);
            }
            $dom->getElementsByTagName('head')->item(0)->appendChild($style);
        }

        $htmlContent = $dom->saveHTML();

        // Récupérer et gérer les erreurs après saveHTML
        $libxmlErrors = libxml_get_errors();
        foreach ($libxmlErrors as $error) {
            // Gérer les erreurs
            if ($error->level === LIBXML_ERR_FATAL) {
                throw new \Exception("Erreur survenue lors de la manipulation du HTML : " . $error->level . " -> " . $error->message);
            }
            continue;
        }

        // Réinitialiser les erreurs libxml
        libxml_clear_errors();
        // Réactivation des erreurs libxml
        libxml_use_internal_errors(false);

        return $htmlContent;
    }

    /**
     * Supprime les images temporaires
     * 
     * @param string $webPath Chemin vers le répertoire web (ex: /var/www/html/public-data/tmp/url_to_pdf/)
     * @return void
     */
    public function deleteTempImages(string $webPath): void
    {
        // Test si $webPath se termine par un slash, si non on le rajoute
        if (substr($webPath, -1) !== '/') {
            $webPath .= '/';
        }

        // Supprime les images temporaires
        if (is_dir($webPath)) {
            $this->rrmdir($webPath);
        }
    }

    private function rrmdir(string $dir): void
    {
        if (is_dir($dir)) {
            $objects = scandir($dir);
            foreach ($objects as $object) {
                if ($object != "." && $object != "..") {
                    if (is_dir($dir . "/" . $object))
                        $this->rrmdir($dir . "/" . $object);
                    else
                        unlink($dir . "/" . $object);
                }
            }
            rmdir($dir);
        }
    }

    /**
     * Supprime un dossier et son contenu m^me si non vide
     * 
     * @param string $dir Chemin vers le répertoire à supprimer
     * @return bool (true si le dossier a été supprimé)
     */
    public function deleteDir(string $dir): bool
    {
        if (!is_dir($dir)) {
            return false;
        }

        $files = array_diff(scandir($dir), ['.', '..']);
        foreach ($files as $file) {
            (is_dir("$dir/$file")) ? $this->deleteDir("$dir/$file") : unlink("$dir/$file");
        }

        if (!rmdir($dir)) {
            throw new \Exception("Le répertoire " . $dir . " n'a pas pu être supprimé.");
        }
        return true;
    }
}
